# ------------------------------------------------------------------------------
# set-up
# ------------------------------------------------------------------------------
library(tidyverse)
library(sdcMicro)

# ------------------------------------------------------------------------------
# load data
# ------------------------------------------------------------------------------
survey_raw <- read_csv(
    here_output("survey_clean_original_labels.csv")
)

siape_raw <- read_csv(
    here_output("siape_clean.csv")
)

estado_raw <- read_csv(
    here_input("estados.csv")
)

survey_weights <- read_csv(
    here_output("survey_weights_ministry_gender_age.csv")
)

# ==============================================================================
# aux
# ==============================================================================
generate_count <- function(data, grouping, name = "total"){
    data %>%
        group_by(
            across(
                all_of(grouping)
            )
        ) %>%
        summarise(
            n = n(),
            .groups = "drop"
        ) %>%
        rename_with(
            ~ name, matches("^n$")
        )
}

hist_total <- function(data, var, ...) ggplot(data) + geom_histogram(aes({{var}}), ...)

here_output <- function(...){here("data", "output", ...)}
here_input <- function(...){here("data", "input", ...)}

# ------------------------------------------------------------------------------
# transform data
# ------------------------------------------------------------------------------
siape <- siape_raw %>%
    rename(
        ministerio = survey_ministry,
        identification_gender = co_sexo
    ) %>%
    mutate(
        identification_gender = str_to_title(identification_gender)
    ) %>% 
    create_edu_bin(co_nivel_escolaridade)

survey <- survey_raw %>%
    create_age_bin(idn_9) %>%
    left_join(
        survey_weights,
        by = c("ministerio", "age_bin", "idn_10" = "identification_gender")
    ) %>%
    create_edu_bin(idn_8) %>%
    anonymize_ministry()

# join estado info
estado <- estado_raw %>%
    mutate(
        across(
            where(is.character),
            convert_to_ascii
        ),
        regiao = str_sub(codigo_uf, 1, 1),
        regiao = recode(
            regiao, 
            `1` = "Norte",
            `2` = "Nordeste",
            `3` = "Sudeste", 
            `4` = "Sul",
            `5` = "Centro-oeste"
        )
    ) %>%
    select(
        codigo_uf, uf, estado = nome, regiao
    )

siape <- siape %>% 
    rename(
        uf = sg_uf_upag
    ) %>% 
    left_join(
        estado,
        by = "uf"
    )

survey <- survey %>% 
    mutate(
        estado = convert_to_ascii(idn_6) %>% 
            # fix pernambuco state
            if_else(. == "pernambuco pernambuco", "pernambuco", .)
    ) %>% 
    left_join(
        estado,
        by = "estado"
    ) %>% 
    rename(
        identification_gender = idn_10
    )

# ==============================================================================
# generate counts
# ==============================================================================
# tally by uf (state), orgao and age
# use five different specifications
identifiers_baseline <- c("ministerio", "regiao")
identifiers_full <- c("ministerio", "regiao", "identification_gender", "edu_bin")

bin <- list(
    baseline = identifiers_baseline,
    full = identifiers_full
)

siape_count <- bin %>% 
    map(
        generate_count,
        data = siape,
        name = "total_siape"
    )

survey_count <- bin %>%
    map(
        generate_count,
        data = survey,
        name = "total_survey"
    )

bin_count_baseline <- siape_count$baseline %>% 
    na.omit() %>%
    inner_join(
        survey_count$baseline
    )

bin_count_full <- siape_count$full %>% 
    na.omit() %>%
    inner_join(
        survey_count$full
    )

# distribution of response
plot_siape <- siape_count %>%
    map(~ hist_total(., total_siape, binwidth = 10))

plot_survey <- survey_count %>% 
    map(hist_total, binwidth = 10)

# ------------------------------------------------------------------------------
# assess risk of identification in survey
# ------------------------------------------------------------------------------
survey_sdc <- survey %>%
    mutate(
        id_survey = row_number(),
        across(all_of(identifiers_full), as.factor)
    )

sdc_out <- createSdcObj(
    dat = survey_sdc,
    keyVars = identifiers_full,
    hhId = "id_survey"
)

freq_table <- sdc_out %>%
    freq(., type = "fk") %>% 
    tibble()

risk_table <- sdc_out@risk$individual %>% 
    as_tibble()

global_risk <- as.data.frame(sdc_out@risk$global)

# ------------------------------------------------------------------------------
# assess risk
# ------------------------------------------------------------------------------
# note that there are two ministries (cidadania, mulher) that have 
# a small count of respondents
bin_count_baseline %>%
    filter(total_siape < 20 | total_survey < 20)

bin_count_full %>% 
    filter(total_siape < 20 | total_survey < 20)

# reecord linkage risk
bin_count_full %>% 
    mutate(
        fk = total_survey/total_siape
    ) %>% 
    filter(fk < 1) %>%
    summarise(
        mean_fk = mean(fk),
        min_fk = min(fk),
        max_fk = max(fk)
    )

global_risk

# ------------------------------------------------------------------------------
# anonymize compromising data
# ------------------------------------------------------------------------------
# only for ministério do meio ambiente, norte
# anonymize region
bins_to_anonymize <- tibble(
    ministerio = c(
        rep("Ministério do Meio Ambiente", 3), 
        "Ministério da Ciência, Tecnologia e Inovações", 
        "Ministério do Meio Ambiente"
    ),
    regiao = c(rep("norte", 3), "nordeste", "sudeste"),
    identification_gender = c("Feminino", rep("Masculino", 4)),
    edu_bin = c(rep("ensino superior", 2), rep("ensino médio", 3)),
    anonymize = 1
)

survey_anonymized <- survey %>% 
    left_join(bins_to_anonymize) %>%
    mutate(
        # recode anonymize
        anonymize = coalesce(anonymize, 0),
        # anomymize DAS
        anonymize = if_else(
            str_detect(idn_7, "DAS"), 1, anonymize
        ),
        # anonymize identifiers for compromised bins and DAS
        across(
            c(all_of(identifiers_full)),
            ~ ifelse(anonymize == 1, NA, .)
        )
    ) %>%
    rename(
        gender = identification_gender,
        commissioned = idn_7
    ) %>%
    select(
        regiao,
        ministerio,
        gender,
        commissioned,
        edu_bin,
        everything(),
        -(deviceid:caseid),
        -(mapa:mtur),
        -starts_with("department"),
        -starts_with("idn"),
        -codigo_uf,
        -estado,
        -uf,
        -anonymize,
        -matches("\\w+_\\d+_\\d+"),
        -matches("rand_sn_\\w+"),
        -rand_sn,
        -fc_text,
        -response,
        -response_rate,
        -siape_total,
        -age_bin
    ) %>%
    # need to remove the open-ended question
    arrange(regiao, ministerio)

# add respondent_id
survey_anonymized <- survey_anonymized %>%
    mutate(
        respondent_id = row_number()
    ) %>%
    select(respondent_id, everything())
    
# ------------------------------------------------------------------------------
# write-out data
# ------------------------------------------------------------------------------
bin_count_baseline %>%
    write_excel_csv(
        here("data", "output", "bins_siape_survey_baseline.csv")
    )

bin_count_full %>%
    write_excel_csv(
        here("data", "output", "bins_siape_survey_full.csv")
    )

survey_anonymized %>% 
    write_excel_csv(
        here("data", "output", "survey_anonymized.csv")
    )
